/*
 * Copyright (c) 2024. Das Land Schleswig-Holstein vertreten durch das Ministerium für Energiewende, Klimaschutz, Umwelt und Natur
 * Zentrales IT-Management
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.admin.common.errorhandling;

import jakarta.validation.ConstraintViolationException;

import org.springframework.data.rest.webmvc.ResourceNotFoundException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ProblemDetail;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.RequiredArgsConstructor;

@RestControllerAdvice
@RequiredArgsConstructor
public class ExceptionController extends ResponseEntityExceptionHandler {

	private final ProblemDetailMapper problemDetailMapper;

	@ExceptionHandler(RuntimeException.class)
	@ResponseBody
	public ProblemDetail handleRuntimeException(RuntimeException ex) {
		return buildProblemDetail(HttpStatus.INTERNAL_SERVER_ERROR, ex);
	}

	@ExceptionHandler(AccessDeniedException.class)
	@ResponseBody
	public ProblemDetail handleAccessDeniedException(AccessDeniedException ex) {
		return buildProblemDetail(HttpStatus.FORBIDDEN, ex);
	}

	@ExceptionHandler(ResourceNotFoundException.class)
	@ResponseBody
	public ProblemDetail handleResourceNotFoundException(ResourceNotFoundException ex) {
		return buildProblemDetail(HttpStatus.NOT_FOUND, ex);
	}

	@ExceptionHandler(FunctionalException.class)
	@ResponseBody
	public ProblemDetail handleFunctionalException(FunctionalException ex) {
		return buildProblemDetail(HttpStatus.BAD_REQUEST, ex);
	}

	@ExceptionHandler(TechnicalException.class)
	@ResponseBody
	public ProblemDetail handleTechnicalException(TechnicalException ex) {
		return buildProblemDetail(HttpStatus.INTERNAL_SERVER_ERROR, ex);
	}

	private ProblemDetail buildProblemDetail(HttpStatus status, Exception ex) {
		return ProblemDetail.forStatusAndDetail(status, ex.getLocalizedMessage());
	}

	@ExceptionHandler(ConstraintViolationException.class)
	public ProblemDetail handleConstraintViolationException(ConstraintViolationException e) {
		return problemDetailMapper.fromConstraintViolationException(e);
	}
}
