package de.ozgcloud.admin.common.errorhandling;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.Set;

import jakarta.validation.ConstraintViolation;
import jakarta.validation.ConstraintViolationException;
import jakarta.validation.Validation;
import jakarta.validation.Validator;
import jakarta.validation.constraints.NotEmpty;

import org.apache.commons.lang3.StringUtils;
import org.assertj.core.util.Arrays;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.ozgcloud.admin.RootController;
import de.ozgcloud.admin.RootModelAssembler;
import de.ozgcloud.common.test.ITCase;
import lombok.Builder;
import lombok.Getter;
import lombok.SneakyThrows;

@ITCase
@AutoConfigureMockMvc
@WithMockUser
class ExceptionControllerITCase {

	@Autowired
	private MockMvc mockMvc;

	@MockBean
	private RootModelAssembler modelAssembler;

	@Nested
	class TestConstraintViolationException {

		@BeforeEach
		void setUpException() {
			when(modelAssembler.toModel(any())).thenAnswer((a) -> {
				throw new ConstraintViolationException(getConstraintViolations());
			});
		}

		private Set<ConstraintViolation<ValidatedClass>> getConstraintViolations() {
			Validator validator = Validation.buildDefaultValidatorFactory().getValidator();
			return validator.validate(ValidatedClass.builder().string(StringUtils.EMPTY).build());
		}

		@Test
		@SneakyThrows
		void shouldHaveInvalidFieldNameInResponse() {
			var result = performGet();

			result.andExpect(jsonPath("$.invalidParams[*].name").value("string"));
		}

		@Test
		@SneakyThrows
		void shouldHaveInvalidParamsInResponse() {
			var result = performGet();

			result.andExpect(jsonPath("$.invalidParams[*].length()").value(Arrays.asList(new Integer[] { 4 })));
		}

		@Test
		@SneakyThrows
		void shouldHaveMessageInResponse() {
			var result = performGet();

			result.andExpect(jsonPath("$.invalidParams[0].reason").value("Empty field"));
		}

		@Test
		@SneakyThrows
		void shouldHaveValueInResponse() {
			var result = performGet();

			result.andExpect(jsonPath("$.invalidParams[0].value").value(""));
		}

		@Getter
		@Builder
		private static class ValidatedClass {
			@NotEmpty(message = "Empty field")
			private String string;
		}

		@SneakyThrows
		private ResultActions performGet() {
			return mockMvc.perform(get(RootController.PATH));
		}
	}
}